/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright held by original author
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

\*---------------------------------------------------------------------------*/

#   include "fluxFactor.H"


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::fluxFactor::fluxFactor
(
    RectangularMatrix<scalar>& fluxFactorTable
)
    :
    fluxFactorTable_(fluxFactorTable),
    indexDistance_i_(0),
    indexDistance_iPlus1_(0),
    f_D_(0.0),
    value_(0.0),
    currentDistanceIndex_(0.0)
{}

// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::fluxFactor::~fluxFactor()
{}

// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

// indexDi: Returns the Distance interpolation factor and updates indexDi_ and indexDiPlus1_
void Foam::fluxFactor::update(scalar Distance_)
{
    if ( (Distance_ < fluxFactorTable_[0][0]) | (Distance_ > fluxFactorTable_[fluxFactorTable_.m() - 1][0]))
    {
        Info << "Distance out of fluxFactor table range" << nl;
        if (Distance_ < fluxFactorTable_[0][0])
        {
            Info << "Distance = "
                 << Distance_
                 << " < "
                 << fluxFactorTable_[0][0]
                 << nl;
        }

        if (Distance_ > fluxFactorTable_[fluxFactorTable_.m() - 1][0])
        {
            Info << "Distance = "
                 << Distance_
                 << " > "
                 << fluxFactorTable_[fluxFactorTable_.m() - 1][0]
                 << nl;
        }

        ::exit(1);
    }

    int i_Distance(0);
    while (fluxFactorTable_[i_Distance][0] < Distance_)
    {
        i_Distance++;
    }

    indexDistance_i_ = i_Distance;

    if (Distance_ == fluxFactorTable_[indexDistance_i_][0])
    {
        indexDistance_iPlus1_ = indexDistance_i_;
        f_D_ = 1;
    }
    else
    {
        indexDistance_iPlus1_ = indexDistance_i_ - 1;
        f_D_ =
            (
                fluxFactorTable_[indexDistance_iPlus1_][0] -
                Distance_
            ) /
            (
                fluxFactorTable_[indexDistance_iPlus1_][0] -
                fluxFactorTable_[indexDistance_i_][0]
            );
    }

    // return f_D_;
    currentDistanceIndex_ = Distance_;
}

Foam::scalar Foam::fluxFactor::fluxValue(scalar Distance_)
{
    if (currentDistanceIndex_ != Distance_)
    {
        update(Distance_);
    }

    return ffactor();
}

Foam::scalar Foam::fluxFactor::pressureValue(scalar Distance_)
{
    if (currentDistanceIndex_ != Distance_)
    {
        update(Distance_);
    }

    return pfactor();
}

// * * * * * * * * * * * * * * * END Member Functions  * * * * * * * * * * * * * //
