//  The advancement of the pyrolysis reactions is computed

// Update pyrolysis zone (nb: if Ta<Tp[i], then there is no pyrolysis at all because the temperature is below the threshold)
forAll(Xsi, i)
{
    forAll(Ta, cellI)
    {
        if (Ta[cellI] < Tp[i].value())
        {
            AField[i][cellI] = 0.0;
        }
        else
        {
            AField[i][cellI] = Ap[i].value();
        }
    }
}

//  Integration
for (int i = 0; i < Xsi.size(); i++)
{
    volScalarField& Xsii = Xsi[i];

    solve
    (
          fvm::ddt(Xsii)
        - fvm::div(mesh.phi(), Xsii)
        - pow(mag(1 - Xsii), mp[i]) * AField[i] * pow(Ta, np[i]) * exp(-Ep[i] / (R * Ta)),
        mesh.solver("Xsii")
    );

    Xsii.min(1.0);

    /* New OpenFoam format

        fvScalarMatrix XsiEqn
        (
        fvm::ddt(Xsii) - fvm::div(mesh.phi(), Xsii)
        - pow((1-Xsii),mp[i])*AField[i]*pow(Ta,np[i])*exp(-Ep[i]/(R*Ta))
            == fvOptions(Xsii)
        );
            XsiEqn.relax();
            fvOptions.constrain(XsiEqn);
            XsiEqn.solve(mesh.solver("Xsii"));
            fvOptions.correct(Xsii);
    */
}

// Compute pyrolysis rate

piTotal = piTotal * 0.;
forAll(pi, i)
{
    pi[i] =
          epsI[hashXsi[i][0]] *
          rhoI[hashXsi[i][0]] *
          Fp[i] *
          (fvc::ddt(Xsi[i]) - fvc::div(mesh.phi(), Xsi[i]));
    piTotal += pi[i];
}


