// semi-implicit resolution of the energy conservation equation
// ('non-conservative' - or developped - form)
GammaHg = ((hg * p * M) / (mu * R * Tg)) * K;
epsgRhogEg =
    (eps_g*rho_g*hg) -
    (eps_g*p); // NOTE: rhog, hg must be updated after advancing the chemistry!!

// explicit computation of the gas temperature to assess the validity of local thermal equilibrium (not yet published)
if (!verifyThermalEquilibrium)
{
    Tg = Ta;
}
else
{
    Tg =
        Ta
      + 1. / Hv *
        (
          - (fvc::ddt(epsgRhogEg) - fvc::div(mesh.phi(), epsgRhogEg))
          + fvc::laplacian(GammaHg, p)
          + Ediff
        );
    // print min and max gas temperatures in the terminal during the run
    // Info << "Tg in " << solidRegions[0].name() << ": max = " << max(Tg).value() << " / min = " << min(Tg).value() << endl;
}

// global energy balance
solve
(
    rhoCp * (fvm::ddt(Ta) - fvm::div(mesh.phi(), Ta))        // storage - implicit in Ta, explicit in rhoCp - with mesh motion correction (ALE)
  + pyrolysisEnergy                                          // storage (second part of the derivative - mass variation) - solid mass loss by pyrolsyis - explicit
  + omegaHeterogeneousEnergy                                 // storage (second part of the derivative - mass variation) - solid mass loss by heterogeneous reactions - explicit
  + fvc::ddt(epsgRhogEg) -  fvc::div(mesh.phi(), epsgRhogEg) // gas storage - explicit
  - fvm::laplacian(k, Ta)                                    // conduction - implicit in Ta, explicit in k
  - fvc::laplacian(GammaHg, p)                               // convection - explicit
  - Ediff                                                    // Energy transported by diffusion of the species - explicit: evaluated in YEqn.H
);

/* print min and max temperatures in the terminal during the run */
Info << "Ta in " << solidRegions[0].name()
     << ": max = " << max(Ta).value()
     << " / min = " << min(Ta).value()
     << endl;

// limiter on Tmin
// forAll(Ta, celli) {if (Ta[celli]<=BL.Tinf(runTime.value())) {Ta[celli]=BL.Tinf(runTime.value());}}
