// Model for a type A fibrous material - may be generalized (see Lachaud2010_JSR)

// update external radius density (matrix or fiber) for matrix-coated fibers being ablated
forAll (rho_ext, i)
{
    if (rT[i] <= rf0.value())
    {
        rho_ext[i] = rhoI[fiberPhase].value();
    }
    else
    {
        rho_ext[i] = rho[matrixPhase][i];
    }
}

forAll (rho_ext.boundaryField()[movingPatchID], faceI)
{
    if (rT.boundaryField()[movingPatchID][faceI] <= rf0.value())
    {
        rho_ext.boundaryFieldRef()[movingPatchID][faceI] =
            rhoI[fiberPhase].value();
    }
    else
    {
        rho_ext.boundaryFieldRef()[movingPatchID][faceI] =
            rho[matrixPhase].boundaryField()[movingPatchID][faceI];
    }
}

// Update solid mass removal by heterogenous reactions
m_dot_ablation = 0. * m_dot_ablation;

forAll(omegaHeterogeneousTotal, i)
{
    omegaHeterogeneousTotal[i] = - chemistry.RR(iCs)[i];
    m_dot_ablation.value() +=
        omegaHeterogeneousTotal[i] *
        mesh.V()[i]; // total in-depth mass loss rate
}

// total radius (fiber + matrix) ablation PDE
solve
(
      fvm::ddt(rT)
    - fvm::div(mesh.phi(), rT)      // radius evolution with ALE correction
    + 1. / (rho_ext * specificSurface) *
      omegaHeterogeneousTotal // recession rate
);


// radius > rff ; part of the fiber is not reactive (ash), it can be removed by erosion

//    rT.max(rff);

// update average geometrical properties

specificSurface =  rT * 2 * (epsI[fiberPhase] + epsI[matrixPhase]) / ( rT0 * rT0);
specificSurface.max(0);

siteDensity =  siteDensityIni * (1. - exp(-pow4(rT / rff))); // +siteDensityIni*10*(1-exp(-pow4(rT/rf0))); // smooth step function centered at rff (rT < rff -> 0, rT > rff -> 1)

forAll(rT, i)
{
    if (rT[i] >= rf0.value())
    {
        eps[matrixPhase][i] =
            epsI[matrixPhase].value() *
            (rT[i] * rT[i] - rf0.value() * rf0.value()) /
            (rT0[i] * rT0[i]  - rf0.value() * rf0.value());
        eps[fiberPhase][i] = epsI[fiberPhase].value();
    }
    else
    {
        eps[matrixPhase][i] = 0;
        eps[fiberPhase][i] =
            epsI[fiberPhase].value() *
            (rT[i] * rT[i]) /
            (rf0.value() * rf0.value());
    }
}

scalarField rTint = rT.boundaryField()[movingPatchID].patchInternalField();
scalarField rT0int = rT0.boundaryField()[movingPatchID].patchInternalField();

forAll(rTint, i)
{
    if (rTint[i] >= rf0.value())
    {
        eps[matrixPhase].boundaryFieldRef()[movingPatchID][i] =
            epsI[matrixPhase].value() *
            (rTint[i] * rTint[i] - rf0.value() * rf0.value()) /
            (rT0int[i] * rT0int[i]  - rf0.value() * rf0.value());
        eps[fiberPhase].boundaryFieldRef()[movingPatchID][i] = epsI[fiberPhase].value();
    }
    else
    {
        eps[matrixPhase].boundaryFieldRef()[movingPatchID][i] = 0. ;
        eps[fiberPhase].boundaryFieldRef()[movingPatchID][i] =
            epsI[fiberPhase].value() *
            (rTint[i] * rTint[i]) /
            (rf0.value() * rf0.value()) ;
    }
}
