// sets the motion of the moving surfaces and updates the mesh
scalarField Tw = Ta.boundaryField()[movingPatchID];
scalarField boundaryPatchSize(Tw.size());
// store surface normal vectorField to the moving patch
vectorField nf =
    - mesh.Sf().boundaryField()[movingPatchID]
    / mesh.magSf().boundaryField()[movingPatchID];

// Grab T wall, p wall, mDotGy wall, and the size of the boundaryPatch
scalarField pw = p.boundaryField()[movingPatchID];
scalarField vw = v.boundaryField()[movingPatchID] & (-nf);
scalarField mDotGyw = mDotGface.boundaryField()[movingPatchID] & (-nf);
scalarField rho_sw = rho_s.boundaryField()[movingPatchID];
scalarField rho_gw = rho_g.boundaryField()[movingPatchID];
scalarField hgBF = hg.boundaryField()[movingPatchID];
scalarField fluxMapw = fluxMap.boundaryField()[movingPatchID];
// Compute B'c (at the wall) and set mesh motion
autoPtr<scalarField> rhoeUeCH_ptr;

if (readBoundaryTecplotTable)
{
    rhoeUeCH_ptr.reset(new scalarField(BL.rhoeUeCH(mesh, movingPatchID)));
}
else
{
    rhoeUeCH_ptr.reset(new scalarField(BL.rhoeUeCH(runTime.value())*fluxMapw)); // if modified here, to be modified in setMotion as well
}
scalarField& rhoeUeCH = rhoeUeCH_ptr();

//*** if(convectiveBoundaryLayer && !volAblation)
if (convectiveBoundaryLayer && !volAblation)
{

// compute data for the meshMotion and for temperature boundary condition (done at the same time to loop only once in the Bprime tables)
    if ((BL.heatOn(runTime.value()) == 1)) // during the heating pulse (heatOn = 1) in convectiveBoundaryLayer mode
    {
        forAll(boundaryPatchSize, i)
        {
            if (mutationBprime)
            {
                Bg_Mutation = max(mDotGyw[i] / (rhoeUeCH[i] * BlowingCorrection.boundaryField()[movingPatchID][i]), 0);
                // update p_Ykg - from wall mass-fraction of element fluxes
                if (((mDotGyw[i]) >= 1e-6) & (elementConservation))
                {
                    scalar p_Ykgt = 0;
                    for (int j = 0; j < ne_Bprime; j++) {
                        scalarField Zsub = Z[j].boundaryField()[movingPatchID].patchInternalField();
                        p_Ykg[j] = (Zsub[i] * mDotGyw[i]) + (ZmassFlux[j].boundaryField()[movingPatchID][i] & (-nf[i]));
                        p_Zsub[j] = Zsub[i]; // store subsurface values (for zeroGradient outlet BC condition)
                        if (p_Ykg[j] <= 0) p_Ykg[j] = 0;
                        p_Ykgt += p_Ykg[j];
                        // (ZmassFlux[j].boundaryField()[movingPatchID][i]&(-nf[i]))/
                        // (ZtotMassFlux.boundaryField()[movingPatchID][i]&(-nf[i]));
                    }
                    for (int j = 0; j < ne_Bprime; j++) {
                        p_Ykg[j] = p_Ykg[j] / p_Ykgt; p_Ykg_BC[j] = p_Ykg[j];
//          Info << "p_Ykg[" << mix.elementName(j) <<"] = " << p_Ykg[j] << nl;
//          Info << "ZmassFlux[" << mix.elementName(j) <<"] = " << Zsub[i]<< nl;
//          Info << "ZtotMassFlux" << mix.elementName(j) <<"] = " <<(ZtotMassFlux.boundaryField()[movingPatchID][i]&(-nf[i])) << nl;
                    }
                }
                else
                {
                    for (int j = 0; j < ne_Bprime; j++)
                    {
                        p_Ykg[j] = p_Z0[j];
                    }
                } // flux null or fixed BC value

                if (!constrainedEquilibrium)
                {
                    mixBprime.surfaceMassBalance
                    (
                        p_Yke,
                        p_Ykg,
                        Tw[i],
                        pw[i],
                        Bg_Mutation,
                        Bc_Mutation,
                        hw_Mutation,
                        p_Xw
                    );
                }
                else // case of O2 in BL is available in testVersion
                {
                    mixBprime.surfaceMassBalance
                    (
                        p_Yke,
                        p_Ykg,
                        Tw[i],
                        pw[i],
                        Bg_Mutation,
                        Bc_Mutation,
                        hw_Mutation,
                        p_Xw
                    );
                    // Info << "Bc ref = " <<  Bc_Mutation << ";  hw = " << hw_Mutation << nl;
                    // Info << "hgw = " << hgBF[i] << nl;
                    // Info << "hcw = " << charred.h(pw[i], Tw[i]) << nl;

                    for (int j = 0; j < Y.size(); j++)
                    {
                        if (Y[j].name() == "CO2" || Y[j].name() == "O2")
                        {
                            // scalar k_constrained = 0.63*Foam::exp(-1160/Tw[i])*Foam::sqrt(8.314*Tw[i]/(2*3.14*32e-3)); // Park O
                            scalar k_constrained = 0.5 * Foam::sqrt(8.314 * Tw[i] / (2 * 3.14 * 32e-3)); // Park    O2
                            // scalar k_constrained = 2.2e-4*Foam::exp(-15500/(8.314*Tw[i])); // Schrooyen  O2
                            k_constrained = 0.2 * k_constrained ; //  // gammaERSA
                            scalar Da = rho_gw[i] * k_constrained / rhoeUeCH[i];
                            scalar invDac = vw[i] / k_constrained;
                            Bc_Mutation = max(1 / (1 + 1 / Da + invDac) * p_Yie[j] * 12 / 16, 1e-10); // note: p_Yie is a pointer (mass fraction), not a pressure.
                        }
                    }
                    // Info << "Bc constrained value = " <<    Bc_Mutation << nl;
                    mixBprime.surfaceMassBalanceConstrained(p_Yke, p_Ykg, Tw[i], pw[i], Bg_Mutation, Bc_Mutation, hw_Mutation, p_Xw);
                    // Info << "Bc constrained out = " <<  Bc_Mutation << ";  hw = " << hw_Mutation << nl;
                }

                // Info << " / Bc mutation = " <<   Bc_Mutation << nl ;
                BprimeCw.boundaryFieldRef()[movingPatchID][i] = Bc_Mutation;
                hw.boundaryFieldRef()[movingPatchID][i] = hw_Mutation;

                // Compute and update the gas mass fractions at the wall
                // 1- compute mass fraction at the wall
                if (elementConservation)
                {
                    double mwg = 0.0;
                    for (int im = 0; im < ne_Bprime; ++im)
                        p_Ykg[im] = 0.0;

                    for (int j = 0; j < ns_Bprime; ++j) {
                        if (mixBprime.species(j).phase() == Mutation::Thermodynamics::GAS)
                        {
                            mwg += mixBprime.speciesMw(j) * p_Xw[j];
                            for (int im = 0; im < ne_Bprime; ++im)
                                p_Ykg[im] += mixBprime.elementMatrix()(j, im) * p_Xw[j];
                        }
                    }

                    for (int im = 0; im < ne_Bprime; ++im)
                    {
                        p_Ykg[im] *= mixBprime.atomicMass(im) / mwg;
                        // Info << "p_Ykg_B'[" << mix.elementName(im) <<"] = " << p_Ykg[im] << nl;
                    }

                    // 2- update element Boundary condition
                    for (int j = 0; j < ne_Bprime; j++) {
                        // if(p_Ykg_BC[j]<=1e-6) {Z[j].boundaryField()[movingPatchID][i] = 0;}  // outlet: zeroGradient
                        // else
                        {Z[j].boundaryFieldRef()[movingPatchID][i] = p_Ykg[j];}     // inlet: fixedValue from B'
                        // Info << "p_Ykg[" << mix.elementName(j) << "] = " << Z[j].boundaryField()[movingPatchID][i] << nl;
                        // Info << "p_Zsub[" << mix.elementName(j) << "] = " << p_Zsub[j] << nl;
                    }

                } // end if(elementConservation)

            } // end mutationBprime

            else
            {
                BprimeCw.boundaryFieldRef()[movingPatchID][i] = BprimeT.BprimeC(pw[i], mDotGyw[i] / (rhoeUeCH[i] * BlowingCorrection.boundaryField()[movingPatchID][i]), Tw[i]);
                hw.boundaryFieldRef()[movingPatchID][i] = BprimeT.hw();
            }
            mDotC.boundaryFieldRef()[movingPatchID][i] = BprimeCw.boundaryField()[movingPatchID][i] * rhoeUeCH[i] * BlowingCorrection.boundaryField()[movingPatchID][i];

            if (fluxMap.boundaryField()[movingPatchID][i] >= fluxThreshold) // imposes motion of the mesh along prescribed direction to conserve topology (may be necessary for large deformation - i.e. lot of ablation)
            {
                Motionw.boundaryFieldRef()[movingPatchID][i] = (BprimeCw.boundaryField()[movingPatchID][i] * rhoeUeCH[i] * BlowingCorrection.boundaryField()[movingPatchID][i]
                        * (1 + failureFraction)
                        + erosionRate)
                        * 1 / rho_sw[i] * (1 / (nf[i] & pointMotionDirection)) * pointMotionDirection;
            }
            else
            {
                Motionw.boundaryFieldRef()[movingPatchID][i] = (BprimeCw.boundaryField()[movingPatchID][i] * rhoeUeCH[i] * BlowingCorrection.boundaryField()[movingPatchID][i]
                        * (1 + failureFraction)
                        + erosionRate)
                        * 1 / rho_sw[i] * nf[i];
            }

        }
    } // end of if ((BL.heatOn(runTime.value())==1))
    else
    {
        // after the heating pluse (cooldown) basic re-radiation without recession, ie: all is 0
        forAll(boundaryPatchSize, i)
        {
            BprimeCw.boundaryFieldRef()[movingPatchID][i] = 0;
            hw.boundaryFieldRef()[movingPatchID][i] = 0;
            mDotC.boundaryFieldRef()[movingPatchID][i] = 0;
            Motionw.boundaryFieldRef()[movingPatchID][i] = 0 * pointMotionDirection;
        }
    }

}// end of "if(convectiveBoundaryLayer & !volAblation)"

//*** if(convectiveBoundaryLayer && volAblation)
if (convectiveBoundaryLayer && volAblation)
{

    grad_rho_s = fvc::grad(rho_s);
    vectorField grad_rho_s_int = grad_rho_s.boundaryField()[movingPatchID].patchInternalField();
    scalarField grad_rho_s_int_proj = grad_rho_s_int & nf;
    grad_rT = fvc::grad(rT);
    vectorField grad_rT_int = grad_rT.boundaryField()[movingPatchID].patchInternalField();
    scalarField grad_rT_int_proj = grad_rT_int & nf;
    scalarField rTw = rT.boundaryField()[movingPatchID].patchInternalField();
    scalarField invDxw = mesh.deltaCoeffs().boundaryField()[movingPatchID];
    scalarField rho_s_int = rho_s.boundaryField()[movingPatchID].patchInternalField();
    scalarField omegaHeterogeneousTotal_int = omegaHeterogeneousTotal.boundaryField()[movingPatchID].patchInternalField();


    forAll(boundaryPatchSize, i)
    {
        if (physicsBasedErosionModel == 1)   // Rate based on min rf and grad(rf)
        {
            Info << "grad_rT_int_proj = " << grad_rT_int_proj << nl;
            if ((rTw[i] < rfFail.value()) & (grad_rT_int_proj[i] > 0))
            {   Motionw.boundaryFieldRef()[movingPatchID][i] = (rfFail.value() - rTw[i]) / (grad_rT_int_proj[i] * runTime.deltaT().value()) * nf[i];
// = (rhoeUeCH[i]/rho_sw[i]) * (rFail - rTw[i])/rFail * nf[i] ;
            }
        }

        if (physicsBasedErosionModel == 2)  // Rate based on density gradient
        {
            if ((rho_sw[i] < 50) & (grad_rho_s_int_proj[i] > 0))
            {   Motionw.boundaryFieldRef()[movingPatchID][i] = (50 - rho_sw[i]) / (grad_rho_s_int_proj[i] * runTime.deltaT().value()) * nf[i];
            }
        }

        if (physicsBasedErosionModel == 3)  // remove cell-by-cell
        {
            if (rTw[i] < rfFail.value())
            {   Motionw.boundaryFieldRef()[movingPatchID][i] = Motionw.boundaryField()[movingPatchID][i] * 0.98 + 0.02 * 1 / (invDxw[i] * runTime.deltaT().value()) * nf[i];
            }
            else {Motionw.boundaryFieldRef()[movingPatchID][i] = 0.98 * Motionw.boundaryField()[movingPatchID][i];}
        }

        if (physicsBasedErosionModel == 4)   // Test smoothing
        {
// Info << "grad_rT_int_proj = " << grad_rT_int_proj << nl;
            if ((rTw[i] < rfFail.value()) ) // & (grad_rT_int_proj[i]>0)
            {   Motionw.boundaryFieldRef()[movingPatchID][i]
                    = max(1.05 * Motionw.boundaryField()[movingPatchID][i] & nf[i], 1e-7 / rho_sw[i]) * nf[i];
                // (rfFail.value() - rTw[i]) / (grad_rT_int_proj[i]*runTime.deltaT().value())
            }
            else {Motionw.boundaryFieldRef()[movingPatchID][i] = 0.95 * Motionw.boundaryField()[movingPatchID][i];}
        }

        if (physicsBasedErosionModel == 5)   // Test omega
        {

            if ((rTw[i] < rfFail.value()) )
            {   Motionw.boundaryFieldRef()[movingPatchID][i]
                    = 0.95 * Motionw.boundaryField()[movingPatchID][i] + 0.05 * omegaHeterogeneousTotal_int[i] / rho_s_int[i] * nf[i];
            }
            else {Motionw.boundaryFieldRef()[movingPatchID][i] = 0.95 * Motionw.boundaryField()[movingPatchID][i];}
        }
    }
}

//*** if(convectiveBoundaryLayer) ="NO"
if (!convectiveBoundaryLayer)
{
    forAll(boundaryPatchSize, i)
    {Motionw.boundaryFieldRef()[movingPatchID][i] = BL.v_abl() * pointMotionDirection;}
}

//*** Special case for numerical analysis - hw is used but motion is forced to be equal to zero
if (!meshMotion)
{
    forAll(boundaryPatchSize, i)
    {
        BprimeCw.boundaryFieldRef()[movingPatchID][i] = 0.0;
        mDotC.boundaryFieldRef()[movingPatchID][i] = 0.0 ;
        Motionw.boundaryFieldRef()[movingPatchID][i] = 0.0 * pointMotionDirection;
    }
}

//*** Mesh motion
motionU.boundaryFieldRef()[movingPatchID] == Motionw.boundaryField()[movingPatchID];
mesh.update();

//  Info << "Recession velocity" << nl << Motionw.boundaryField()[movingPatchID] << nl;
#include "volContinuity.H" // checks the topology of the mesh after the motion
