// read and store the boundary layer properties into the RAM for faster access
Info << "Reading boundary layer properties" << nl;

Switch readBoundaryTecplotTable = setCase.lookupOrDefault<Switch>("readBoundaryTecplotTable", "no");

autoPtr<BLinput> BL_ptr;
autoPtr<RectangularMatrix<scalar> > BLinputTable_ptr;

if (!readBoundaryTecplotTable)
{
    //  OFstream BLoutputFile ("BLout");  // opens an output file
    IFstream BLinputFile(
        "constant/environmentProperties/boundaryConditions");  // opens an input file
    IFstream BLinputFileTemp(
        "constant/environmentProperties/boundaryConditions");  // opens a temporary input file

    if (BLinputFile.good() == false) // checks that the input file is opened
    {
        Info << "boundaryLayer data not found" << nl;
        exit (1); // exits otherwise
    }

    Info << "The boundaryLayer file must have 13 columns: "
         "t(s), p_w(Pa), T_w(K), v_abl(m/s), rhoUeCH(kg/m²/s), "
         "h_r(J/kg), q_rad(W/m²), q_other(W/m²), heatOn, lambda, "
         "T_inf(K), pressureFactorOn, p_dyn_stag (Pa)"
         << nl;

    int columnTable = 13;
    int rawTable = 0;
    scalar temp, rawTableFrac, rawTableInt;
    int i_raw = 0;

    while (true)
    {
        BLinputFileTemp >> temp;
        if (BLinputFileTemp.eof() == 1) {break;}
        i_raw++;
    }

    rawTableFrac =
        modf
        (
            static_cast<scalar>(i_raw) / static_cast<scalar>(columnTable),
            &rawTableInt
        );

    if (rawTableFrac != 0) {
        Info << "... and it does not seem to have 13 columns." << nl;
        Info << rawTableInt << " lines and "
             << rawTableFrac * columnTable << " 'double precision' have been read."
             << nl;
        exit(1);
    }
    else
    {
        rawTable = i_raw / columnTable;
        Info << "The boundary-layer data file has been read (" << rawTable
             << " lines)." << nl;
    }

    BLinputTable_ptr.reset(new RectangularMatrix<scalar>(rawTable, columnTable));
    RectangularMatrix<scalar>& BLinputTable = BLinputTable_ptr();

    for (int x = 0; x < rawTable; x++)
    {
        for (int i = 0; i < columnTable; i++)
        {
            BLinputFile >> BLinputTable[x][i];
            // BLoutputFile << BLinputTable[x][i] << " ";
        }
        // BLoutputFile << nl;
    }
}
else
{
    BLinputTable_ptr.reset(new RectangularMatrix<scalar>(2, 13));
}
BL_ptr.reset(new BLinput(BLinputTable_ptr(), readBoundaryTecplotTable, runTime, solidRegions));

// BL table is an object of the class BLinput, which handles the interpolation for any of the variables when called
BLinput& BL = BL_ptr();

// the call format is BL.variable(time);
Info << "The boundary layer properties have been read." << nl;
//  Info << "Process killed after reading the boundary layer properties table." << nl; return (0);  // exits without error
// END OF THE IO TEST
