// opening OpenFOAM output file streams for write.H

mkdir("output", 0777);
OFstream thermocouple("output/thermocouple");
OFstream gasTemperature("output/gasTemperature");
OFstream pressure("output/pressure");
OFstream density("output/density");
OFstream mass("output/mass");
OFstream delta_t("output/delta_t");
OFstream surfacePatchFace0 ("output/surfacePatchFace0");

// creating headers for the output files
scalarList probe_list(setOutput.lookup("oneDprobeLocations"));
int probe_list_size = probe_list.size();
scalar probe[probe_list_size + 1];

for (int i = 0; i < probe_list_size; i++)
{
    probe[i] = probe_list[i];
}

delta_t << "t (s) "
        << "	"
        << "delta_t (s)"
        << "	"
        << "Tw (K)"
        << "	"
        << "p (Pa)"
        << "	"
        << "m_dot_g_surf (kg/m²/s)"
        << "	"
        << "m_dot_c (kg/m²/s)"
        << "	"
        << "rho_s (kg/m³)"
        << "	"
        << "recession (m)" << endl;

if (oneDoutput)
{
    // Temperature
    thermocouple << "t (s) "
                 << "	"
                 << "surface";

    for (int i = 0; i < probe_list_size; i++)
    {
        thermocouple << "	" << probe[i] << "m";
    }

    thermocouple << "	"
                 << "bottom"
                 << nl;

    // gasTemperature
    gasTemperature << "t (s) "
                   << "	"
                   << "surface" ;

    for (int i = 0; i < probe_list_size; i++)
    {
        gasTemperature << "	" << probe[i] << "m";
    }

    gasTemperature << "	"
                   << "bottom"
                   << nl;

    // pressure
    pressure << "t (s) "
             << "	"
             << "surface" ;

    for (int i = 0; i < probe_list_size; i++)
    {
        pressure << "	" << probe[i] << "m";
    }

    pressure << "	"
             << "bottom"
             << nl;

    // density
    density << "t (s) "
            << "	"
            << "surface" ;

    for (int i = 0; i < probe_list_size; i++)
    {
        density << "	" << probe[i] << "m";
    }

    density << "	"
            << "bottom"
            << nl;

    // Mass-related data
    mass << "t (s) "
         << "	"
         << "m_dot_g_surf (kg/m²/s)"
         << "	"
         << "m_dot_c (kg/m²/s)"
         << "	"
         << "< 0.98 Virgin"
         << "	"
         << "> 1.02 Char"
         << "	"
         << "Total recession (m)"
         << nl;
    mass << "0.0"
         << "	"
         << 0.0
         << "	"
         << 0.0
         << "	"
         << 0.0
         << "	"
         << 0.0
         << "	"
         << 0.0
         << nl;

    // Time step
} // if (oneDoutput)
else
{
    // surfacePatchFace0 data
    surfacePatchFace0 << "t (s) "
                      << "	"
                      << "Tw (K)"
                      << "	"
                      << "p (Pa)"
                      << "	"
                      << "m_dot_g_surf (kg/m²/s)"
                      << "	"
                      << "m_dot_c (kg/m²/s)"
                      << "	"
                      << "rho_s (kg/m³)"
                      << "	"
                      << "recession (m)"
                      << nl;
} // if (oneDoutput)

/* Open string to name profile files */

int plotIndex = 0;
std::stringstream sstrmXi;
std::stringstream sstrmrF;
std::stringstream sstrmp;
std::stringstream sstrmT;
std::stringstream sstrmYi;
std::stringstream sstrmPlotYi;
std::stringstream sstrmPlotH;
std::stringstream sstrmProbe;
std::stringstream sstrmTprobe;
std::stringstream sstrmShape;
std::stringstream sstrmSGF;

// grab initial thickness of moving patch for data processing of 1D cases
scalar initialThickness = 0 ;
if (mesh.Cf().boundaryField()[movingPatchID].size() > 0)
    initialThickness = mesh.Cf().boundaryField()[movingPatchID][0][1];

// index list of Cf in increasing distance from Cf0
int patchSize = mesh.Cf().boundaryField()[movingPatchID].size();
int CfdistanceIndex[patchSize]; //first index is true index, second index is ranking

//first 'reference' face is Cf0
for (int i = 0; i < patchSize; i++)
    CfdistanceIndex[i] = i;

int j = 0;
scalar minD = 0;
scalar minD_ref = 0;

for (int i = 1; i < patchSize; i++)
{
    if (minD_ref < (mag(mesh.Cf().boundaryField()[movingPatchID][i] -
                        mesh.Cf().boundaryField()[movingPatchID][0])))
    {
        CfdistanceIndex[i] = i;
        minD_ref = mag(mesh.Cf().boundaryField()[movingPatchID][i] -
                       mesh.Cf().boundaryField()[movingPatchID][0]);
    }
    else
    {
        j = i;
        minD = minD_ref;
        while (minD > (mag(mesh.Cf().boundaryField()[movingPatchID][i] -
                           mesh.Cf().boundaryField()[movingPatchID][0])))
        {
            CfdistanceIndex[i] = CfdistanceIndex[i] - 1;
            CfdistanceIndex[j-1] = CfdistanceIndex[j-1] + 1;
            j--;
            minD = mag(mesh.Cf().boundaryField()[movingPatchID][j-1] -
                       mesh.Cf().boundaryField()[movingPatchID][0]);
        }
    }
}

