/*---------------------------------------------------------------------------*\
write.H

Description
    Output fields and data of interest for post-processing.
\*---------------------------------------------------------------------------*/

// Single processor run options. Output processed data files on the whole mesh.
// Note : in parallel, only master processor mesh data are readily available before reconstruction.
if (!Pstream::parRun()) // if not parallel run
{
    if (oneDoutput) // if unidimensional case
    {
        // Output in a file called "mass" mass-related data in the format required
        // for the Ablation test cases: "pyrolysis front" and "mass loss rates"
        if (runTime.outputTime())
        {
            int rhoSize = rho_s.size() - 1;

            scalar rho_c_2 = rho_c + 0.02 * (rho_v - rho_c);
            scalar rho_v_98 = rho_c + 0.98 * (rho_v - rho_c);

            if ((rho_s[rhoSize] >= rho_v_98) || (Xsi.size() < 1))
            {
                scalar mDotC_patch0 = 0;
                if (!volAblation) {mDotC_patch0 = mDotC.boundaryField()[movingPatchID][0];}
                else {mDotC_patch0 = m_dot_ablation.value() / mesh.magSf().boundaryField()[movingPatchID][0];}
                mass << runTime.timeName() << " " << mDotGface.boundaryField()[movingPatchID][0][1] << "    " << mDotC_patch0 << "  " << 0 << " " << 0 << " " << initialThickness - mesh.Cf().boundaryField()[movingPatchID][0][1] << nl;
            }
            else
            {
                scalar char_front = mesh.C()[rhoSize][1];
                scalar virgin_front = mesh.C()[rhoSize][1];

                int index_rho = 0;
                while (rho_s[index_rho] > rho_v_98)
                {
                    index_rho++;
                }
                if (rho_s[index_rho] == rho_v_98)
                {virgin_front = mesh.C()[index_rho][1];}
                else
                {
                    scalar weight_front = (rho_s[index_rho] - rho_v_98) / (rho_s[index_rho] - rho_s[index_rho - 1]);
                    virgin_front = (1 - weight_front) * mesh.C()[index_rho][1] + weight_front * mesh.C()[index_rho - 1][1];
                }
                if (rho_s[rhoSize] < rho_c_2)
                {
                    index_rho--;
                    while (rho_s[index_rho] > rho_c_2)
                    {
                        index_rho++;
                    }
                    if (rho_s[index_rho] == rho_c_2)
                    {char_front = mesh.C()[index_rho][1];}
                    else
                    {
                        scalar weight_front = (rho_s[index_rho] - rho_c_2) / (rho_s[index_rho] - rho_s[index_rho - 1]);
                        char_front = (1 - weight_front) * mesh.C()[index_rho][1] + weight_front * mesh.C()[index_rho - 1][1];
                    }
                }

                scalar mDotC_patch0 = 0;
                if (!volAblation) {mDotC_patch0 = mDotC.boundaryField()[movingPatchID][0];}
                else {mDotC_patch0 = m_dot_ablation.value() / mesh.magSf().boundaryField()[movingPatchID][0];}

                mass <<  runTime.timeName() << "    " << mDotGface.boundaryField()[movingPatchID][0][1] << "    " << mDotC_patch0 << "  " << initialThickness - virgin_front << "   " << initialThickness - char_front << " " << initialThickness - mesh.Cf().boundaryField()[movingPatchID][0][1] << nl;

            }
        }
// end mass data

// Output the temperature at fixed positions in the sample (ie, "thermocouple values") as required for the ablation test cases
        if (runTime.outputTime() || runTime.timeName() == "0")
        {
            scalar Tprobe[probe_list_size + 1];
            scalar pprobe[probe_list_size + 1];
            scalar rhoprobe[probe_list_size + 1];
            scalar Tgprobe[probe_list_size + 1];
            scalar Tweight = 0.;

            int Tpsize = Ta.size() - 1;
            int index_Tp = Tpsize;

            for (int probeIndex = 0; probeIndex < probe_list_size; probeIndex++)
            {
                if (mesh.C().boundaryField()[movingPatchID][0][1] < probe[probeIndex])
                {
                    Tprobe[probeIndex] = -0.0001;
                }
                else
                {
                    if (mesh.C()[Tpsize][1] <= probe[probeIndex])
                    {
                        Tweight =
                            (mesh.C()[Tpsize][1] - probe[probeIndex]) /
                            (mesh.C()[Tpsize][1] - mesh.C().boundaryField()[movingPatchID][0][1]);
                        Tprobe[probeIndex] =
                            (1. - Tweight) * Ta.internalField()[Tpsize] +
                            Tweight * Ta.boundaryField()[movingPatchID][0];
                        pprobe[probeIndex] =
                            (1. - Tweight) * p.internalField()[Tpsize] +
                            Tweight * p.boundaryField()[movingPatchID][0];
                        rhoprobe[probeIndex] =
                            (1. - Tweight) * rho_s.internalField()[Tpsize] +
                            Tweight * rho_s.boundaryField()[movingPatchID][0];
                        Tgprobe[probeIndex] =
                            (1. - Tweight) * Tg.internalField()[Tpsize] +
                            Tweight * Tg.boundaryField()[movingPatchID][0];
                    }
                    else
                    {
                        while (mesh.C()[index_Tp][1] > probe[probeIndex])
                        {
                            index_Tp--;
                        }

                        if (mesh.C()[index_Tp][1] == probe[probeIndex])
                        {
                            Tprobe[probeIndex] = T[index_Tp];
                        }
                        else
                        {
                            Tweight =
                                (mesh.C()[index_Tp][1] - probe[probeIndex]) /
                                (mesh.C()[index_Tp][1] - mesh.C()[index_Tp + 1][1]);
                            Tprobe[probeIndex] =
                                (1. - Tweight) * Ta.internalField()[index_Tp] +
                                Tweight * Ta.internalField()[index_Tp + 1];
                            pprobe[probeIndex] =
                                (1. - Tweight) * p.internalField()[index_Tp] +
                                Tweight * p.internalField()[index_Tp + 1];
                            rhoprobe[probeIndex] =
                                (1. - Tweight) * rho_s.internalField()[index_Tp] +
                                Tweight * rho_s.internalField()[index_Tp + 1];
                            Tgprobe[probeIndex] =
                                (1. - Tweight) * Tg.internalField()[index_Tp] +
                                Tweight * Tg.internalField()[index_Tp + 1];
                        }
                    }
                }
            }

            // write
            thermocouple << runTime.timeName() << " "
                         << Ta.boundaryField()[movingPatchID][0];
            for (int probeIndex = 0; probeIndex < probe_list_size; probeIndex++)
                thermocouple << " " << Tprobe[probeIndex] ;
            thermocouple << "   " << Ta.internalField()[0] << " " << endl;

            gasTemperature << runTime.timeName() << "   "
                           << Tg.boundaryField()[movingPatchID][0];
            for (int probeIndex = 0; probeIndex < probe_list_size; probeIndex++)
                gasTemperature << "   " << Tgprobe[probeIndex];
            gasTemperature << " " << Tg.internalField()[0] << " " << endl;

            pressure << runTime.timeName() << " "
                     << p.boundaryField()[movingPatchID][0];
            for (int probeIndex = 0; probeIndex < probe_list_size; probeIndex++)
                pressure << " " << pprobe[probeIndex];
            pressure << "   " << p.internalField()[0] << "  " << endl;

            density << runTime.timeName() << "  "
                    << rho_s.boundaryField()[movingPatchID][0];
            for (int probeIndex = 0; probeIndex < probe_list_size; probeIndex++)
                density << "  " << rhoprobe[probeIndex];
            density << "    " << rho_s.internalField()[0] << "  " << endl;
        }
        // end output thermocouples

        // PRINT data PROFILES for 1D CASES
        if ((runTime.outputTime() || runTime.timeName() == "0") & plotProfiles)
        {
            if (runTime.timeName() == "0")
            {
                mkdir("output/profiles", 0777);
                mkdir("plots/profiles", 0777);
            }

            sstrmXi << "output/profiles/rhos" << runTime.timeName();
            sstrmrF << "output/profiles/hg" << runTime.timeName();
            sstrmp << "output/profiles/p" << runTime.timeName();
            sstrmT << "output/profiles/T" << runTime.timeName();
            sstrmYi << "output/profiles/Yi" << runTime.timeName();
            sstrmPlotYi << "plots/profiles/plotYi_" << runTime.timeName();
            sstrmPlotH << "plots/profiles/plotH_" << runTime.timeName();
            plotIndex++;
            OFstream rhosProfile (sstrmXi.str().c_str());  // opens an output file
            rhosProfile << " location(m) " << " " << "rho_s (kg/m³)" <<  nl;

            OFstream hgProfile (sstrmrF.str().c_str());  // opens an output file
            hgProfile << " location(m) " << "   " << "hg (J/kg)" <<  nl;

            OFstream pProfile (sstrmp.str().c_str());  // opens an output file
            pProfile << " location(m) " << "    " << "p (Pa)" <<  nl;

            OFstream TProfile (sstrmT.str().c_str());  // opens an output file
            TProfile << " location(m) " << "    " << "T (K)" <<  nl;
            OFstream YiProfile (sstrmYi.str().c_str());  // opens an output file

            if (finiteRate)
            {
                YiProfile << " location(m) ";
                for (label i = 0; i < Y.size(); i++)
                    YiProfile << "    " <<  Y[i].name();
                YiProfile << nl;
            }
            else
            {
                YiProfile << "Species mass-fraction profiles are only "
                             "provided in finiteRate mode."
                          << nl;
            }

            OFstream plotYi (sstrmPlotYi.str().c_str());  // opens an output file
            OFstream plotH (sstrmPlotH.str().c_str());  // opens an output file

            // Get rid of the current content ..
            sstrmXi.str("");
            sstrmrF.str("");
            sstrmp.str("");
            sstrmT.str("");
            sstrmYi.str("");
            sstrmPlotYi.str("");
            sstrmPlotH.str("");

            // Print density profile
            rhosProfile << "-------------------------------------" << nl;
            rhosProfile << "t = " << runTime.timeName() << " s" << nl;
            rhosProfile << "-------------------------------------" << nl;
            rhosProfile << initialThickness - mesh.Cf().boundaryField()[movingPatchID][0][1] << "   " << rho_s.boundaryField()[movingPatchID][0] << nl;
            int rhossize = rho_s.size() - 1;
            forAll(rho_s, i)
            {
                rhosProfile << initialThickness - mesh.C()[rhossize - i][1] << "    "
                            << rho_s[rhossize - i] << "   "
                            << nl;
            }

            // Print hg profile
            hgProfile << "-------------------------------------" << nl;
            hgProfile << "t = " << runTime.timeName() << " s" << "  " << "h_g (J/kg)" << "  " << "h_bar (J/kg)" << "    " << "T (K)" << nl;
            hgProfile << "-------------------------------------" << nl;
            hgProfile << 0 << " " << hg.boundaryField()[movingPatchID][0] <<  " " << h_bar.boundaryField()[movingPatchID][0] << "   " << Ta.boundaryField()[movingPatchID][0] << nl;
            int hgsize = hg.size() - 1;

            forAll(hg, i)
            {
                hgProfile << mesh.Cf().boundaryField()[movingPatchID][0][1] - mesh.C()[hgsize - i][1] << "  "
                          << hg[hgsize - i] << "    "
                          << h_bar[hgsize - i] << " "
                          << Ta[hgsize - i]
                          << nl;
            }

            plotH << "set terminal pdf enhanced size 6,4" << nl;
            plotH << "set xlabel 'y (m)' " << nl;
            plotH << "set ylabel 'Enthalpies (J/kg)' " << nl;
            plotH << "set key inside" << nl;
            plotH << "set autoscale x" << nl;
            plotH << "set autoscale y" << nl;
            plotH << "set output 'plotH_"   << runTime.timeName() << ".pdf' " << nl;
            plotH << "set title 'Enthalpies profile at t = "  << runTime.timeName() << " s'" << nl;
            plotH << "plot " << "\\" << nl;
            plotH <<  "'../../output/profiles/hg" <<  runTime.timeName() << "' using 1:2  with lines lw 2 title 'hg_finite-rate' ,\\" << nl;
            plotH <<  "'../../output/profiles/hg" <<  runTime.timeName() << "' using 1:3  with lines lw 2 title 'hg_equilibrium' ,\\" <<  nl;
            plotH <<  "'../../output/profiles/hg" <<  runTime.timeName() << "' using 1:4  with lines lw 2 title 'h_bar'"  << nl;

            // Print p profile
            pProfile << "-------------------------------------" << nl;
            pProfile << "t = " << runTime.timeName() << " s" << nl;
            pProfile << "-------------------------------------" << nl;
            pProfile << 0 << "  " << p.boundaryField()[movingPatchID][0] << nl;
            int psize = p.size() - 1;
            forAll(p, i)
            {
                pProfile << mesh.Cf().boundaryField()[movingPatchID][0][1] - mesh.C()[psize - i][1] << "    " << p[psize - i] << "  " << nl;
            }

            // Print T profile
            TProfile << "-------------------------------------" << nl;
            TProfile << "t = " << runTime.timeName() << " s" << nl;
            TProfile << "-------------------------------------" << nl;
            TProfile << mesh.Cf().boundaryField()[movingPatchID][0][1] << " " << Ta.boundaryField()[movingPatchID][0] << nl;
            int Tsize = Ta.size() - 1;
            forAll(Ta, i)
            {
                TProfile <<  mesh.C()[Tsize - i][1] << "    " << Ta[Tsize - i] << " " << nl;
            }


            /* Print Yi profile */
            if (finiteRate) {  // only in finiteRate mode
                YiProfile << "-------------------------------------" << nl;
                YiProfile << "t = " << runTime.timeName() << " s" << nl;
                YiProfile << "-------------------------------------" << nl;
                YiProfile << 0;

                for (label i = 0; i < Y.size(); i++)
                {
                    YiProfile << "    " <<  Y[i].boundaryField()[movingPatchID][0];
                }
                YiProfile << nl;

                forAll (p, celli)
                {
                    YiProfile << mesh.Cf().boundaryField()[movingPatchID][0][1] - mesh.C()[psize - celli][1];
                    for (label i = 0; i < Y.size(); i++)
                    {
                        YiProfile << "    " << Y[i].internalField()[psize - celli];
                    }

                    YiProfile << nl;
                }

                plotYi << "set terminal pdf enhanced size 6,4" << nl;
                plotYi << "set xlabel 'y (m)' " << nl;
                plotYi << "set ylabel 'Mass fractions' " << nl;
                plotYi << "set key outside" << nl;
                plotYi << "set autoscale x" << nl;
                plotYi << "set yrange [0.0001:1]" << nl;
                plotYi << "set logscale y" << nl;
                plotYi << "set output 'plotYi_"   << runTime.timeName() << ".pdf' " << nl;
                plotYi << "set title 'Test-case 1.0. Species mass-fraction profile at t = "  << runTime.timeName() << " s'" << nl;
                plotYi << "plot " << "\\" << nl;

                for (label i = 0; i < Y.size() - 1; i++)
                {
                    plotYi <<  "'../../output/profiles/Yi" <<  runTime.timeName() << "' using 1:" << i + 2 << " title '" << Y[i].name() << "' ,\\" << nl;
                }
                plotYi <<  "'../../output/profiles/Yi" <<  runTime.timeName() << "' using 1:" << Y.size() + 1 << " title '" << Y[Y.size() - 1].name() << "'" << nl;
            } // end of print Yi profile in finiteRate mode
        }
        // END OF PRINT PROFILES
    } // end of output for 1D geometries (oneDoutput)

    // Output surface properties from patch 0 (stag. point).
    if (runTime.outputTime() || runTime.timeName() == "0")
    {
        surfacePatchFace0 << runTime.timeName() << "    "
                          << Ta.boundaryField()[movingPatchID][0] << " "
                          << p.boundaryField()[movingPatchID][0] << "  "
                          << mDotGface.boundaryField()[movingPatchID][0][1] << "   "
                          << mDotC.boundaryField()[movingPatchID][0] << "  "
                          << rho_s.boundaryField()[movingPatchID][0] << "  "
                          << initialThickness - mesh.Cf().boundaryField()[movingPatchID][0][1]
                          << endl;
    }

    // print time steps, and other data at each time step
    delta_t << runTime.timeName() << "  "
            << runTime.deltaT().value() << "   "
            << Ta.boundaryField()[movingPatchID][0] << "   "
            << p.boundaryField()[movingPatchID][0] << "    "
            << mDotGface.boundaryField()[movingPatchID][0][1] << " "
            << mDotC.boundaryField()[movingPatchID][0] << "    "
            << rho_s.boundaryField()[movingPatchID][0] << "    "
            << initialThickness - mesh.Cf().boundaryField()[movingPatchID][0][1]
            << endl;
    // end print time steps

    // Output external shape
    if ((runTime.outputTime() || runTime.timeName() == "0") && plotShape)
    {
        if (runTime.timeName() == "0")
            mkdir("output/shape", 0777);
        sstrmShape << "output/shape/shape_" << runTime.timeName();
        OFstream shape (sstrmShape.str().c_str());  // opens an output file
        shape << " x (m) " << " " << "y (m)" <<  nl;
        sstrmShape.str("");
        scalarField Tw = Ta.boundaryField()[movingPatchID];
        scalarField boundaryPatchSize(Tw.size());

        // Print shape
        shape << "-------------------------------------" << nl;
        shape << "t = " << runTime.timeName() << " s" << nl;
        shape << "-------------------------------------" << nl;
        forAll(boundaryPatchSize, faceI)
        {
            shape << mesh.Cf().boundaryField()[movingPatchID][faceI][0] << "    "
                  << mesh.Cf().boundaryField()[movingPatchID][faceI][1]
                  << nl;
        }
    }

    // Output gas flow on moving patch along curvilinear coordinate
    if ((runTime.outputTime() || runTime.timeName() == "0") && plotMdotGmag)
    {
        if (runTime.timeName() == "0")
            mkdir("output/MdotGmag", 0777);
        sstrmSGF << "output/MdotGmag/MdotGmag_" << runTime.timeName();
        OFstream SGF (sstrmSGF.str().c_str());  // opens an output file
        SGF << " s (m) " << "   " << "surface gas flow (kg/m2/s)" <<  nl;
        sstrmSGF.str("");
        scalarField Tw = Ta.boundaryField()[movingPatchID];
        // Print SGF
        SGF << "-------------------------------------" << nl;
        SGF << "t = " << runTime.timeName() << " s" << nl;
        SGF << "-------------------------------------" << nl;
        scalar s = mesh.Cf().boundaryField()[movingPatchID][0][0]; // s for first Cell center (x direction)
        SGF << s << "   " << mag(mDotGface.boundaryField()[movingPatchID][0]) << nl;
        vectorField nf =
            - mesh.Sf().boundaryField()[movingPatchID] /
              mesh.magSf().boundaryField()[movingPatchID];
        scalarField mDotGywBF = mDotGface.boundaryField()[movingPatchID] & (-nf);

        for (int faceI = 1; faceI < Tw.size(); faceI++)
        {
            s =
                s +
                mag(mesh.Cf().boundaryField()[movingPatchID][CfdistanceIndex[faceI]] -
                    mesh.Cf().boundaryField()[movingPatchID][CfdistanceIndex[faceI - 1]]);
            SGF << s << "   "
                << mag(mDotGface.boundaryField()[movingPatchID][CfdistanceIndex[faceI]]) << "  "
                << mDotGywBF[CfdistanceIndex[faceI]]
                << nl;
        }
    }
} // end of if !parallelRun

