
        // set the boundary conditions on the porous material surface in the case of non coupled simulations
	if (!cfd)
	{
        #include "setPressureBC.H" 	// surface pressure
        #include "setMotion.H"          // surface motion
        #include "setEnergyBC.H"   	// surface temperature
	}
	else
	{
        #include "setMotion.H"          // surface motion
	}

        // compute the pyrolysis rate
        Info << "Advancing pyrolysis" << nl;
        #include "pyrolysis.H"

        // compute gas and solid properties
        Info << "Updating data fields" << nl;
        #include "computeData.H"

        // solve gas mass-momentum conservation equation
	if (porous)
	{
        Info << "Gass mass conservation" << nl;
        #include "gMass.H"
	}

        // solve finite-rate chemistry and species conservation
        if (finiteRate)
        {
            Info << "Computing chemistry rates" << nl;
            #include "chemistry.H"

            if (volAblation)
            {
                Info << "Solid mass conservation" << nl;
                #include "sMass.H" // solve solid-mass conservation equation (in-depth ablation)
                #include "setSpeciesBC.H"
            }

            Info << "Species conservation" << nl;
            #include "YEqn.H" // solve gas species mass-fraction conservation equation
        }

        // solve element conservation (in equilibrium mode only)
        else
        {
            if (elementConservation)
            {
                // update wall element composition (from B') and solve element conservation equation
                #include "setElementsBCgrad.H"
                Info << "Element conservation" << nl;
                #include "ZEqn.H"
            }
        }

        // solve energy conservation equations
        Info << "Energy conservation" << nl;
        #include "energy2T.H"           // porous material
        #include "energySubStructure.H" // substructure materials

        // output solid data files
        #include "write.H"

