/*---------------------------------------------------------------------------*\
ZEqn.H

Description
    Element conservation equation (equilibrium chemistry mode).
\*---------------------------------------------------------------------------*/

epsphi  = mDotGface & mesh.Sf(); // Mass flux of pyrolysis gases.
epsrhog = eps_g * rho_g;

tmp<fv::convectionScheme<scalar> > mvConvection
(
    fv::convectionScheme<scalar>::New
    (
        mesh,
        fields,
        epsphi,
        mesh.divScheme("div(epsphi,Zi)")
    )
);


{
    volScalarField Zt = 0.0 * Z[0];
    ZtotMassFlux = 0.0 * ZmassFlux[0];

    for (label i = 0; i < Z.size(); i++)
    {
        volScalarField& Zi = Z[i];
        volScalarField& Dmi = Dm[i];
        volScalarField totalZp = piTotal;
        totalZp = 0. * totalZp;
        forAll(Xsi, j)
        {
            totalZp += Zp[j][i] * pi[j];
        }

        fvScalarMatrix ZiEqn
        (
              fvm::ddt(epsrhog, Zi)                    // storage
            // - fvm::div(fvc::interpolate(epsrhog)*mesh.phi(), Zi)  // mesh motion correction (ALE) done in mesh.solver
            + mvConvection->fvmDiv(epsphi, Zi)         // convection
            - fvm::laplacian(Dmi / eta0 * epsrhog, Zi) // diffusion
            - totalZp                 // elements produced by the pyrolysis reactions
            == fvOptions(epsrhog, Zi)
        );

        ZiEqn.relax();
        fvOptions.constrain(ZiEqn);
        ZiEqn.solve(mesh.solver("Zi"));
        fvOptions.correct(Zi);

        Zi.max(0.0);
        Zt += Zi;

        // compute the mass diffusion fluxes of the elements
        // convection - implement as zeroGradient, ie, we need the sub surface cell value at the boundary.
        ZmassFlux[i] =  -  fvc::interpolate(Dmi / eta0 * epsrhog) * (mesh.Sf() / mesh.magSf()) * fvc::snGrad(Zi); //diffusion
        ZtotMassFlux += ZmassFlux[i]; // average mass diffusion flux
    }

    for (label i = 0; i < Z.size(); i++)
        Z[i] /= Zt;
}

/*  vectorField nf =
        - mesh.Sf().boundaryField()[movingPatchID]
        /mesh.magSf().boundaryField()[movingPatchID];
    forAll(Z[i].boundaryField()[movingPatchID], i)
    {
            for(int j=0; j<ne_mix; j++){
            Info << "ZmassFlux[" << mix.elementName(j) <<"] = " << (ZmassFlux[j].boundaryField()[movingPatchID][i]&(-nf[i])) << nl;
            }
    }
*/
// compute the energy (flux) transported by diffusion of the molecules
/*
 Ediff = Ediff0;

// speciesHOverRT(Ta, hSpeciesCell)

 for(label i=0; i<Y.size(); i++)
    {
        volScalarField& Dmi = Dm[i];
        volScalarField& hSpeciesFieldi = hSpeciesField[i];

    // energy (enthalpy) transported by molecular diffusion
    Ediff = Ediff + fvc::laplacian(Dmi/eta0*epsrhog*hSpeciesFieldi, Y[i]);
    }
*/
/* to print the enthalpy of the mixture in J/kg as a function of T (in K)
for (int tvar = 0 ; tvar < 20 ; tvar ++ ){
scalar enth=0;
    for(label i=0; i<Y.size(); i++)
    {
enth = enth + Y[i][0] * chemistry.hsi(i, 200 + tvar *100);
    }
Info << "h[T= " << 200 + tvar * 100 << "] =" << enth << nl;
}
*/
